@@ -1,5 +1,6 @@
 # Copyright (c) 2011, SD Elements. See LICENSE.txt for details.
 
+import json
 import logging
 from re import compile
 
@@ -10,7 +11,7 @@ from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import reverse, resolve
 from django.http import HttpResponseRedirect, HttpResponse
 from django.test.signals import setting_changed
-from django.utils import simplejson as json, timezone
+from django.utils import timezone
 import django.views.static
 
 from password_expiry import password_is_expired
@@ -61,7 +62,7 @@ class DoNotTrackMiddleware:
     The parameter can take True, False or None values based on the presence of
     the ``Do Not Track`` HTTP header in client's request, which in turn depends
     on browser's configuration. The header indicates client's general preference
-    to opt-out from behavioral profiling and third-party tracking. 
+    to opt-out from behavioral profiling and third-party tracking.
 
     The parameter does **not** change Django's behaviour in any way as its sole
     purpose is to pass the user's preference to application and it's up to the owner
@@ -114,7 +115,7 @@ class XssProtectMiddleware(BaseMiddleware):
       ``on``         enable full XSS filter blocking XSS requests (may `leak document.referrer <http://homakov.blogspot.com/2013/02/hacking-with-xss-auditor.html>_`)
       ``off``        completely disable XSS filter
 
-    Reference: 
+    Reference:
 
     - `Controlling the XSS Filter <http://blogs.msdn.com/b/ieinternals/archive/2011/01/31/controlling-the-internet-explorer-xss-filter-with-the-x-xss-protection-http-header.aspx>`_
     """
@@ -152,8 +153,8 @@ class ContentNoSniff:
     where web page would for example load a script that was disguised as an user-
     supplied image.
 
-    Reference: 
-    
+    Reference:
+
     - `MIME-Handling Change: X-Content-Type-Options: nosniff <http://msdn.microsoft.com/en-us/library/ie/gg622941(v=vs.85).aspx>`_
     """
 
@@ -233,7 +234,7 @@ class NoConfidentialCachingMiddleware(BaseMiddleware):
 
     .. _cache_control: https://docs.djangoproject.com/en/dev/topics/cache/#controlling-cache-using-other-headers
 
-    Reference: 
+    Reference:
 
     - `HTTP/1.1 Header definitions - What is Cacheable <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.9.1>`_
     """
@@ -288,32 +289,56 @@ class XFrameOptionsMiddleware(BaseMiddleware):
       - ``allow-from URL``  allow frames from specified *URL*
 
     **Note:** Frames and inline frames are frequently used by ads, social media
-    plugins and similar widgets so test these features after setting this flag. For
-    more granular control use ContentSecurityPolicyMiddleware_.
+    plugins and similar widgets so test these features after setting this flag.
 
-    References: 
-    
-    - `RFC 7034: HTTP Header Field X-Frame-Options <http://tools.ietf.org/html/rfc7034>`_
+    You can exclude certain URLs from this header by setting
+    ``X_FRAME_OPTIONS_EXCLUDE_URLS`` to a list of URL regexes like so::
+
+        X_FRAME_OPTIONS_EXCLUDE_URLS = (
+            r'^/some/url/here$',        # Note the initial slash
+            r'^/another/to/exclude$',
+        )
+
+    The header will be sent unless ``request.path`` matches any of the above list.
+    For more granular control, use ContentSecurityPolicyMiddleware_.
+
+    References:
+
+      - `RFC 7034: HTTP Header Field X-Frame-Options <http://tools.ietf.org/html/rfc7034>`_
     """
 
-    OPTIONAL_SETTINGS = ('X_FRAME_OPTIONS',)
+    OPTIONAL_SETTINGS = ('X_FRAME_OPTIONS', 'X_FRAME_OPTIONS_EXCLUDE_URLS')
 
     DEFAULT = 'deny'
 
     def load_setting(self, setting, value):
-        if not value:
-            self.option = XFrameOptionsMiddleware.DEFAULT
-            return
-        value = value.lower()
-        if value not in ['sameorigin', 'deny'] and not value.startswith('allow-from:'):
-            raise ImproperlyConfigured(XFrameOptionsMiddleware.__name__+" invalid option for X_FRAME_OPTIONS.")
-        self.option = value
+        if setting == 'X_FRAME_OPTIONS':
+            if value:
+                value = value.lower()
+                if value not in ['sameorigin', 'deny'] and not value.startswith('allow-from:'):
+                    raise ImproperlyConfigured(self.__class__.__name__+" invalid option for X_FRAME_OPTIONS.")
+                self.option = value
+            else:
+                self.option = XFrameOptionsMiddleware.DEFAULT
+        elif setting == 'X_FRAME_OPTIONS_EXCLUDE_URLS':
+            if value:
+                try:
+                    self.exclude_urls = [compile(url) for url in value]
+                except TypeError:
+                    raise ImproperlyConfigured(self.__class__.__name__+" invalid option for X_FRAME_OPTIONS_EXCLUDE_URLS")
+            else:
+                self.exclude_urls = []
 
     def process_response(self, request, response):
         """
         And X-Frame-Options and Frame-Options to the response header.
         """
-        response['X-Frame-Options'] = self.option
+        for url in self.exclude_urls:
+            if url.match(request.path):
+                break
+        else:
+            response['X-Frame-Options'] = self.option
+
         return response
 
 # preserve older django-security API
@@ -518,8 +543,8 @@ class StrictTransportSecurityMiddleware:
       - ``STS_MAX_AGE``               time in seconds to preserve host's STS policy (default: 1 year)
       - ``STS_INCLUDE_SUBDOMAINS``    True if subdomains should be covered by the policy as well (default: True)
 
-    Reference: 
-    
+    Reference:
+
     - `HTTP Strict Transport Security (HSTS) <https://datatracker.ietf.org/doc/rfc6797/>`_
     """
 
@@ -553,7 +578,7 @@ class P3PPolicyMiddleware(BaseMiddleware):
     **Note:** P3P work stopped in 2002 and the only popular
     browser with **limited** P3P support is MSIE.
 
-    Reference: 
+    Reference:
 
     - `The Platform for Privacy Preferences 1.0 (P3P1.0) Specification - The Compact Policies <http://www.w3.org/TR/P3P/#compact_policies>`_
     """
